package com.rjconsultores.ventaboletos.web.gui.controladores.gr;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Intbox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Aidf;
import com.rjconsultores.ventaboletos.entidad.AidfEspecie;
import com.rjconsultores.ventaboletos.entidad.AidfTipo;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.service.AidfService;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaAidfController")
@Scope("prototype")
public class BusquedaAidfController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private AidfService aidfService;
	
	@Autowired
	private EstadoService estadoService;
	
	@Autowired
    private transient PagedListWrapper<Aidf> plwAidf;
	private Paging pagingAidf;
	
	private MyListbox aidfList;
	
	private List<AidfEspecie> lsAidfEspecie;
	private Combobox cmbAidfEspecie;
	
	private List<Estado> lsEstado;
	private Combobox cmbEstado;
	
	private List<AidfTipo> lsAidfTipo;
	private Combobox cmbAidfTipo;
	
	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;
	
	private Textbox txtDocFiscal;
	private Textbox txtSerie;
	private Intbox txtAidf;

	
	@Override
	public void doAfterCompose(Component comp) throws Exception {		
				
		lsAidfTipo = aidfService.obtenerTodosAdifTipo();
		lsAidfEspecie = aidfService.obtenerTodosAdifEspecie();
		lsEstado = estadoService.obtenerTodos();
		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();
		
		super.doAfterCompose(comp);
		
		aidfList.setItemRenderer(new AidfListItemRenderer());
		aidfList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Aidf aidf = (Aidf) aidfList.getSelected();
				visualizaAidf(aidf);
			}
		});
		
		actualizaAidfList();
	}
	
	private void visualizaAidf(Aidf _aidf){
		
		if (_aidf == null)
			return;
		
		Map args = new HashMap();
		args.put("aidf", _aidf);
		args.put("aidfList", aidfList);

		openWindow("/gui/gr/editarAidf.zul",
				Labels.getLabel("editarAidfController.window.title"), args, MODAL);
		
	}
	
	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		actualizaAidfList();
	}

	public void onClick$btnRefresh(Event ev) {
		actualizaAidfList();
	}

	public void onClick$btnNovo(Event ev) {
		visualizaAidf(new Aidf());
	}
	
	private void actualizaAidfList() {
		HibernateSearchObject<Aidf> aidfBusqueda =
				new HibernateSearchObject<Aidf>(Aidf.class, pagingAidf.getPageSize());

		aidfBusqueda.addFilterEqual(ACTIVO, true);

		
		Comboitem cbiEspecie = cmbAidfEspecie.getSelectedItem();
		if (cbiEspecie != null) {
			AidfEspecie especie = (AidfEspecie) cbiEspecie.getValue();
			aidfBusqueda.addFilterEqual("aidfEspecie", especie);
		}

		Comboitem cbiTipo = cmbAidfTipo.getSelectedItem();
		if (cbiTipo != null) {
			AidfTipo tipo = (AidfTipo) cbiTipo.getValue();
			aidfBusqueda.addFilterEqual("aidfTipo", tipo);
		}
				
		Comboitem cbiEstado = cmbEstado.getSelectedItem();
		if (cbiEstado != null) {
			Estado estado = (Estado) cbiEstado.getValue();
			aidfBusqueda.addFilterEqual("estado", estado);
		}
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();		
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			aidfBusqueda.addFilterEqual("empresa", empresa);
		} else {
			aidfBusqueda.addFilterIn("empresa", lsEmpresas);
		}
		
		if(!StringUtils.isBlank(txtDocFiscal.getValue())){
			aidfBusqueda.addFilterLike("docfiscal", txtDocFiscal.getValue());
		}
		
		if(!StringUtils.isBlank(txtSerie.getValue())){
			aidfBusqueda.addFilterLike("serie", txtSerie.getValue());
		}
		
		if(txtAidf.getValue() != null){
			aidfBusqueda.addFilterEqual("aidfId", txtAidf.getValue());
		}
		
		aidfBusqueda.addSortDesc("fecvencimiento");
		aidfBusqueda.addSortDesc("aidfId");

		plwAidf.init(aidfBusqueda, aidfList, pagingAidf);
		
		if (aidfList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaAidfController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public List<AidfEspecie> getLsAidfEspecie() {
		return lsAidfEspecie;
	}

	public void setLsAidfEspecie(List<AidfEspecie> _lsAidfEspecie) {
		lsAidfEspecie = _lsAidfEspecie;
	}

	public List<Estado> getLsEstado() {
		return lsEstado;
	}

	public void setLsEstado(List<Estado> _lsEstado) {
		lsEstado = _lsEstado;
	}

	public List<AidfTipo> getLsAidfTipo() {
		return lsAidfTipo;
	}

	public void setLsAidfTipo(List<AidfTipo> _lsAidfTipo) {
		lsAidfTipo = _lsAidfTipo;
	}

	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> _lsEmpresas) {
		lsEmpresas = _lsEmpresas;
	}

	public Textbox getTxtDocFiscal() {
		return txtDocFiscal;
	}

	public void setTxtDocFiscal(Textbox txtDocFiscal) {
		this.txtDocFiscal = txtDocFiscal;
	}

	public Textbox getTxtSerie() {
		return txtSerie;
	}

	public void setTxtSerie(Textbox txtSerie) {
		this.txtSerie = txtSerie;
	}	
	
}
